//
//  KPCell.m
//  TableViewPlayground
//
//  Created by Kalle Persson on 05/10/13.
//  Copyright (c) 2013 Kalle Persson. All rights reserved.
//

#import "KPCell.h"
#import "KPCellAccessoryView.h"

@interface KPCell()

@property (nonatomic) CGFloat panOffset;

@end

@implementation KPCell

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    if (!(self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]))
        return nil;
    
    return self;
}

- (CGFloat)accessoryViewWidth
{
    return 90.0;
}

- (CGFloat)panThreshold
{
    return 0.85;
}

- (void)setLeftAccessoryView:(KPCellAccessoryView *)leftAccessoryView
{
    _leftAccessoryView = leftAccessoryView;
    [self.contentView addSubview:_leftAccessoryView];
}

- (void)setRightAccessoryView:(KPCellAccessoryView *)rightAccessoryView
{
    _rightAccessoryView = rightAccessoryView;
    [self.contentView addSubview:_rightAccessoryView];
}

- (void)setTextField:(UITextField *)textField
{
    _textField = textField;
    [self.contentView addSubview:_textField];
}

- (void)layoutSubviews
{
    [super layoutSubviews];
    self.textField.frame = CGRectMake(15.0, 0, CGRectGetWidth(self.contentView.frame), CGRectGetHeight(self.contentView.frame));
    self.leftAccessoryView.frame = [self frameForAccessoryViewWithXOffset:-[self accessoryViewWidth]];
    self.rightAccessoryView.frame = [self frameForAccessoryViewWithXOffset:CGRectGetWidth(self.contentView.frame)];
}

- (void)prepareForReuse
{
    [self.textField removeFromSuperview];
    [self.leftAccessoryView removeFromSuperview];
    [self.rightAccessoryView removeFromSuperview];
    [super prepareForReuse];
}

#pragma mark - Panning methods

- (void)setOffset:(CGFloat)offset;
{
    _panOffset = offset;
    
    if(offset > [self accessoryViewWidth]) {
        CGFloat diff = offset - [self accessoryViewWidth];
        self.leftAccessoryView.frame = [self frameForAccessoryViewWithXOffset:-[self accessoryViewWidth] - diff];
    } else if (offset < -[self accessoryViewWidth]) {
        CGFloat diff = -[self accessoryViewWidth] - offset;
        self.rightAccessoryView.frame = [self frameForAccessoryViewWithXOffset:CGRectGetWidth(self.contentView.frame) + diff];
    }
    
    self.leftAccessoryView.alpha = MAX(offset / ([self accessoryViewWidth] * [self panThreshold]), 0.1);
    self.rightAccessoryView.alpha = MAX(offset / (-[self accessoryViewWidth] * [self panThreshold]), 0.1);
    
    self.leftAccessoryView.state = self.panOffset > [self accessoryViewWidth] * [self panThreshold] ?KPCellAccessoryViewStateActive : KPCellAccessoryViewStateInactive;
    
    self.rightAccessoryView.state = self.panOffset < -[self accessoryViewWidth] * [self panThreshold] ?KPCellAccessoryViewStateActive : KPCellAccessoryViewStateInactive;

    self.contentView.frame = CGRectMake(offset, 0, CGRectGetWidth(self.frame), CGRectGetHeight(self.frame));
}

- (void)panEndedAtIndexPath:(NSIndexPath *)indexPath {
    if (self.panOffset > [self accessoryViewWidth] * [self panThreshold])
        [self.delegate cell:self atIndexPath:indexPath didActivateLeftAccessoryView:self.leftAccessoryView];
    
    if (self.panOffset < -[self accessoryViewWidth] * [self panThreshold])
        [self.delegate cell:self atIndexPath:indexPath didActivateRightAccessoryView:self.rightAccessoryView];
}

- (void)beganPan
{
    // TODO: Currently isn't being triggered when state is UIGestureRecognizerStateBegan. Investigate
    [UIView animateWithDuration:0.0 animations:^(void) {
         self.contentView.frame = CGRectMake(0, 0, CGRectGetWidth(self.contentView.frame), CGRectGetHeight(self.contentView.frame));
    }];
}

- (void)resetOffset {
    self.panOffset = 0.0;
    
    
    [UIView animateWithDuration:0.6 delay:0 usingSpringWithDamping:0.6 initialSpringVelocity:0 options:0 animations:^(void){
        self.contentView.frame = CGRectMake(0, 0, CGRectGetWidth(self.contentView.frame), CGRectGetHeight(self.contentView.frame));
    } completion:^(BOOL finished) {
        self.leftAccessoryView.state = KPCellAccessoryViewStateInactive;
        self.rightAccessoryView.state = KPCellAccessoryViewStateInactive;
    }];
}

- (CGRect)frameForAccessoryViewWithXOffset:(CGFloat)xOffset
{
    return CGRectMake(xOffset, 0, [self accessoryViewWidth], CGRectGetHeight(self.contentView.frame));
}

@end
