//
//  KPItem.m
//  TableViewPlayground
//
//  Created by Kalle Persson on 21/10/13.
//  Copyright (c) 2013 Kalle Persson. All rights reserved.
//

#import "KPItem.h"

NSString * const KPItemTitleKey = @"title";
NSString * const KPItemStatusKey = @"status";

@implementation KPItem

#pragma mark - NSCoder delegate methods

+ (KPItem *)itemWithTitle:(NSString *)title
{
    KPItem *item = [[KPItem alloc] init];
    
    item.title = title;
    item.status = KPItemStatusDefault;
    
    return item;
}

- (id)initWithCoder:(NSCoder *)decoder
{
    if(!(self = [super init]))
        return nil;
    
    self.title = [decoder decodeObjectForKey:KPItemTitleKey];
    
    NSNumber *statusNumber = [decoder decodeObjectForKey:KPItemStatusKey];
    switch (statusNumber.integerValue) {
        case KPItemStatusArchived:
            self.status = KPItemStatusArchived;
            break;
            
        case KPItemStatusFavourited:
            self.status = KPItemStatusFavourited;
            break;
            
        default:
            self.status = KPItemStatusDefault;
            break;
    }
    
    return self;
}

- (void)encodeWithCoder:(NSCoder *)encoder
{
    [encoder encodeObject:self.title forKey:KPItemTitleKey];
    [encoder encodeObject:@(self.status) forKey:KPItemStatusKey];    
}

- (void)archive {
    self.status = KPItemStatusArchived;
}

- (void)unarchive {
    self.status = KPItemStatusDefault;
}

- (void)favourite {
    self.status = KPItemStatusFavourited;
}

- (void)unFavourite {
    self.status = KPItemStatusDefault;
}

- (BOOL)isArchived
{
    return self.status == KPItemStatusArchived;
}

- (BOOL)isFavourited
{
    return self.status == KPItemStatusFavourited;
}

-(NSString *)description
{
    return [NSString stringWithFormat:@"Title: %@, Status: %d", self.title, self.status];
}

@end
