//
//  KPViewController.m
//  TableViewPlayground
//
//  Created by Kalle Persson on 05/10/13.
//  Copyright (c) 2013 Kalle Persson. All rights reserved.
//

#import "KPListViewController.h"
#import "KPDatasource.h"
#import "KPCell.h"
#import "KPTableView.h"

@interface KPListViewController () <UITableViewDelegate, UIGestureRecognizerDelegate, KPDatasourceDelegate, KPCellDelegate, UITextFieldDelegate, UIScrollViewDelegate>

@property (nonatomic) KPItemCategory category;
@property (strong, nonatomic) UIPanGestureRecognizer *panGestureRecognizer;
@property (strong, nonatomic) UITableView *tableView;
@property (strong, nonatomic) KPDatasource *dataSource;
@property (strong, nonatomic) KPCell *currentPanningCell;
@property (nonatomic) CGPoint panStart;
@property (nonatomic) CGFloat panOffset;
@property (nonatomic) BOOL canSwipe;
@property (nonatomic) BOOL didJustAddNewRow;
@property (nonatomic) CGFloat scrollStartY;

@end

@implementation KPListViewController

- (id)init
{
    return [self initWithCategory:KPItemCategoryInbox];
}

- (id)initWithCategory:(KPItemCategory)category
{
    if (!(self = [super init]))
        return nil;
    
    _category = category;
    
    return self;
}

- (NSString *)title {
    switch (self.category) {
        case KPItemCategoryInbox:
            return @"Inbox";
        case KPItemCategoryArchived:
            return @"Archived";
        case KPItemCategoryFavourited:
            return @"Favourited";
        case KPItemCategoryCategoryCount:
            return nil;
    }
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    

    /*
     CGFloat barHeight = 60.0;
    self.navBar = [[UINavigationBar alloc] initWithFrame:CGRectMake(0, 0, CGRectGetWidth(self.view.frame), barHeight)];
    UINavigationItem *title = [[UINavigationItem alloc] initWithTitle:[self title]];
    UIBarButtonItem *addItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(addRow)];
    title.rightBarButtonItem = addItem;
    self.navBar.items = @[title];
    [self.view addSubview:self.navBar];
     self.toolbarItems = @[[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(addRow)]];
     */
    
    
    
    
    self.tableView = [[KPTableView alloc] initWithFrame:CGRectMake(0, 0, CGRectGetWidth(self.view.frame), CGRectGetHeight(self.view.frame))];
    
    
    self.dataSource = (KPDatasource *)[[KPDatasource alloc] initWithCategory:self.category];
    self.dataSource.delegate = self;

    self.tableView.delegate = self;
    [self.view addSubview:self.tableView];
    
    self.tableView.dataSource = self.dataSource;
    
    self.panGestureRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(didPanCell:)];
    [self.tableView addGestureRecognizer:self.panGestureRecognizer];
}

- (void)didPanCell:(UIPanGestureRecognizer *)panGestureRecognizer
{
    switch (panGestureRecognizer.state) {
        case UIGestureRecognizerStateBegan:
        {
            CGPoint location = [panGestureRecognizer locationOfTouch:0 inView:self.tableView];
            NSIndexPath * indexPath = [self.tableView indexPathForRowAtPoint:location];

            if (![self.dataSource tableView:self.tableView canSwipeRowAtIndexPath:indexPath]) {
                self.currentPanningCell = nil;
                return;
            }
            
            KPCell *cell = (KPCell *)[self.tableView cellForRowAtIndexPath:indexPath];
            self.currentPanningCell = cell;
            self.panStart = location;
            [self.currentPanningCell beganPan];
        }
        case UIGestureRecognizerStateChanged:
        {
            CGPoint location = [panGestureRecognizer locationOfTouch:0 inView:self.tableView];
            if (!self.currentPanningCell)
                return;
            
            CGFloat panOffset = (self.panStart.x - location.x) * -1;
            [self.currentPanningCell setOffset:panOffset];
            CGFloat cellHeight = [self tableView:self.tableView heightForRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
            
            if (ABS(panOffset) > cellHeight * .75)
                self.tableView.scrollEnabled = NO;
            
            break;
        }
            
        case UIGestureRecognizerStateEnded:
        {
            if (!self.currentPanningCell)
                return;
            [self.currentPanningCell panEndedAtIndexPath:[self.tableView indexPathForCell:self.currentPanningCell]];
            [self.currentPanningCell resetOffset];
            self.tableView.scrollEnabled = YES;
            break;
        }
        case UIGestureRecognizerStateCancelled:
        {
            [self.currentPanningCell resetOffset];
            self.tableView.scrollEnabled = YES;
            break;
        }
            
        default:
            break;
    }
}

#pragma mark - UITableViewDelegate methods


- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPatha
{
    return 55.0;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.row < [self.dataSource tableView:tableView actualNumberOfRowsInSection:indexPath.section])
        return;
    
    NSArray *indexPathsForAllRows = [self.dataSource indexPathsForAllRowsInSection:indexPath.section includeDeleteRow:YES];
    [self.tableView beginUpdates];
    [self.dataSource archiveAllRows];
    [self.tableView deleteRowsAtIndexPaths:indexPathsForAllRows withRowAnimation:UITableViewRowAnimationFade];
    [self.tableView endUpdates];
}

#pragma mark - KPCellDelegate methods
- (void)cell:(KPCell *)cell atIndexPath:(NSIndexPath *)indexPath didActivateLeftAccessoryView:(UIView *)accessoryView
{
    [self.tableView beginUpdates];
    if (self.category == KPItemCategoryArchived) {
        [self.dataSource toggleArchiveRowAtIndexPath:indexPath];
    } else {
        [self.dataSource toggleArchiveRowAtIndexPath:indexPath];
    }
    [self.tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    [self.tableView endUpdates];
}

- (void)cell:(KPCell *)cell atIndexPath:(NSIndexPath *)indexPath didActivateRightAccessoryView:(UIView *)accessoryView
{
    if (self.category == KPItemCategoryArchived)
        return;
    
    [self.tableView beginUpdates];
    [self.dataSource toggleFavouriteRowAtIndexPath:indexPath];
    if (self.category == KPItemCategoryFavourited) {
        [self.tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];       
    } else {
        [self.tableView reloadRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }
    [self.tableView endUpdates];
}

- (void)addRow
{
    NSIndexPath *newRowIndexpath = [NSIndexPath indexPathForRow:0 inSection:0];
    //NSIndexPath *deleteAllIndexPath = [NSIndexPath indexPathForRow:1 inSection:0];
    
    [self.tableView beginUpdates];
    
    [self.dataSource insertRowAtIndex:newRowIndexpath.row];
    [self.tableView insertRowsAtIndexPaths:@[newRowIndexpath] withRowAnimation:UITableViewRowAnimationBottom];
    [self.tableView endUpdates];
    
    KPCell *cell = (KPCell *)[self.tableView cellForRowAtIndexPath:newRowIndexpath];
    [cell.textField becomeFirstResponder];
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    self.didJustAddNewRow = NO;
    [textField resignFirstResponder];
    return YES;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
    NSString *text = textField.text;
    KPCell *cell = (KPCell *)textField.superview.superview.superview;
    NSIndexPath *indexPath = [self.tableView indexPathForCell:cell];
    [self.dataSource updateText:text forRowAtIndexPath:indexPath];
}

#pragma mark - UIScrollViewDelegate

- (void)scrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate
{
    self.canSwipe = YES;
    self.scrollStartY = 0;
}

- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
    self.didJustAddNewRow = NO;
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    self.canSwipe = NO;
    self.scrollStartY = scrollView.contentOffset.y;
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView
{
    if (!self.didJustAddNewRow || scrollView.contentOffset.y > 64)
        [self.view endEditing:YES];
    
    if (scrollView.contentOffset.y < -144 && !self.didJustAddNewRow) {
        [self addRow];
        self.didJustAddNewRow = YES;
        [self.tableView.panGestureRecognizer cancelsTouchesInView];
    }
    
    CGFloat verticalThreshold = [self tableView:self.tableView heightForRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]] / 2;

    if (ABS(scrollView.contentOffset.y - self.scrollStartY) > verticalThreshold) {
        [self.panGestureRecognizer setEnabled:NO];
        [self.panGestureRecognizer setEnabled:YES];
    }
    
}


@end
