//
//  KPRootViewController.m
//  TableViewPlayground
//
//  Created by Kalle Persson on 06/11/13.
//  Copyright (c) 2013 Kalle Persson. All rights reserved.
//

#import "KPRootViewController.h"
#import "KPListViewController.h"
#import "KPDatasource.h"

NSString * const kRootViewCellReuseIdentifier = @"rootViewCell";

@interface KPRootViewController () <UITableViewDelegate, UITableViewDataSource>

@property (nonatomic, strong) NSArray *rootItems;

@end

@implementation KPRootViewController

- (id)init
{
    if (!(self = [super init]))
        return nil;
    
    _rootItems = @[@"Inbox",
                   @"Archived items",
                   @"Favourited items"];
    
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kRootViewCellReuseIdentifier];
    NSUInteger row = indexPath.row;
    
    if (!cell)
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:kRootViewCellReuseIdentifier];
    
    if (row < [self.rootItems count])
        cell.textLabel.text = self.rootItems[row];
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (indexPath.row >= KPItemCategoryCategoryCount)
        return;
    
    KPListViewController *viewController = [[KPListViewController alloc] initWithCategory:indexPath.row];
    [self.navigationController pushViewController:viewController animated:YES];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [self.rootItems count];
}

- (BOOL)shouldAutorotate
{
    return YES;
}

@end
